;*********************************************************************
; COMMODORE VIC 20 BOOT USING BASIC 2.0
; written by Robert Hurst <robert@hurst-ri.us>
; updated version: 31-Oct-2010
;
; Modified 2023 by thegg to incorporate changes by beamrider to improve
; the display of multicolour sprites and 24 bit high sprites. The modification
; is for assembly using CBM prg Studio.
;       
; 10 SYS (4640)

*=$1201

        BYTE    $0E, $12, $0A, $00, $9E, $20, $28,  $34, $36, $34, $30, $29, $00, $00, $00


;*********************************************************************
; Starting entry point for this program
;
;                .segment "STARTUP"
*=$1220

MAIN 
                LDX $FFFC
                LDY $FFFD
                STX $0318
                STY $0319           ; enable RESTORE key as RESET
                LDA MACHINE
                CMP #$05
                BNE PALF
                ;
                ; NTSC setup
NTSC            LDX #<NTSCT         ; load the timer low-byte latches
                LDY #>NTSCT
                LDA #$78            ; raster line 240
                BNE IRQSYNCF
NTSCt = $4243                   ; (261 * 65 - 2)
                ;
                ; PAL setup
PALF            LDX #<PALT          ; load the timer low-byte latches
                LDY #>PALT
                LDA #$82            ; raster line 260
PALT = $5686                    ; (312 * 71 - 2)
                ;
IRQSYNCF 
                CMP VIC+$04
                BNE IRQSYNCF
                STX $9126        ; load T1 latch low
                STY $9125        ; load T1 latch high, and transfer both to T1 counter


;*********************************************************************
; Now that all the VIC startup initialization stuff is completed,
; you can append one-time startup code/data here, i.e., like a splash
; title screen.  Then, you must jump to your CODE segment, linked
; outside of VIC's internal RAM address space ...
;
RUNONCE 
                ;
                ;====  INSERT ANY ADDITIONAL STARTUP CODE HERE  ====
                ;
                JMP RESTART

;*********************************************************************
; VIC Software Sprite Stack 2010 (VIC-SSS-MMX)
;
; The above BASIC loader will be overwritten by SSS upon its
; initialization (SSSINIT).  The linker will fill this reserved space
; with values used for the dual video frame buffers, play field, and
; the sprite image buffers and registers: 4096 - 6207 ($1000 - $1BFF)
;
; $1000 - $11FF         VICFRAME1 - first video buffer
; $1200 - $13FF         VICFRAME2 - second video buffer
; $1400 - $15FF         PLAYFIELD - write-pending screen buffer
; $1600 - $17FF         PLAYCOLOR - write-pending color / dirty buffer
; $1800 - $1BFF         Sprite image buffers & registers
;

*=$1800
SSSBUF          bytes 512         ; this can be resized as required --
                                  ; if all 64-chars are used by sprites, that
                                  ; exhausts all 128 custom characters for
                                  ; double-buffering (x2)
;1A00 - 1bff register space
;
; SPRITE REGISTERS (17)
;
SPRITEBACK      bytes SPRITEMAX ; 1st char this sprite is in collision with
SPRITEBUFH      bytes SPRITEMAX ; pointer within sprite image buffer
SPRITEBUFL      bytes SPRITEMAX ; pointer within sprite image buffer
SPRITEC1H       bytes SPRITEMAX ; pointer within sprite display character pool
SPRITEC1L       bytes SPRITEMAX ; pointer within sprite display character pool
SPRITEC2H       bytes SPRITEMAX ; pointer within sprite display character pool
SPRITEC2L       bytes SPRITEMAX ; pointer within sprite display character pool
SPRITECOL       bytes SPRITEMAX ; 4-bit VIC color code
SPRITECX        bytes SPRITEMAX ; sprite collision X-coord
SPRITECY        bytes SPRITEMAX ; sprite collision Y-coord
SPRITEDEF       bytes SPRITEMAX ; function/matrix definition (see explanation below)
SPRITEDEF2      bytes SPRITEMAX ; extended sprite definition
SPRITEH         bytes SPRITEMAX ; number of raster lines (1-16)
SPRITEIMGH      bytes SPRITEMAX ; pointer to source graphic for rendering at 0,0
SPRITEIMGL      bytes SPRITEMAX ; pointer to source graphic for rendering at 0,0
SPRITEX         bytes SPRITEMAX ; horizontal pixel coordinate, visible >0 - <SSSCLIPX
SPRITEY         bytes SPRITEMAX ; vertical pixel coordinate, visible >0 - <SSSCLIPY
SPRITEZ         bytes SPRITEMAX ; bit 0: last rendered (0 = SPRITEC1; 1 = SPRITEC2)
                                ; bit 1: fast copy (0 = merge; 1 = copy)
                                ; bit 3: sprite collision
                                ; bit 4: sprite image is clipped by a static cell
                                ; bit 5: background is all SSSNULLs
                                ; bit 6: copy/merge into alternate sprite character pool
                                ; bit 7: copy/shift sprite image into its buffer
SSSZORDR        bytes SPRITEMAX

;SPRITEDEF              ; matrix definition:
                                 ; bit 0: unused
                                 ; bit 1: unused
                                 ; bit 2: unused
                                 ; bit 3: unused
                                 ; bit 4: repeat         0=independent; 1=re-use previous
                                 ; bit 5: ghost          0=merge image; 1=invert image
                                 ; bit 6: collision      0=ignore; 1=detect
                                 ; bit 7: enabled        0=invisible; 1=visible

;SPRITEDEF2             ; matrix definition:
                                 ; bit 0: unused
                                 ; bit 1: width          0 = 8px; 1 = 16px
                                 ; bit 2: float Y        0=fixed cell; 1=vertical float
                                 ; bit 3: float X        0=fixed cell; 1=horizontal float
                                 ; bit 4: height         00 = 8, 01 =16, 10 = 24, 11=Unsupported
                                 ; bit 5: height         00 = 8, 01 =16, 10 = 24, 11=Unsupported
                                 ; bit 6: unused
                                 ; bit 7: unused

*=$1b30
SPRITES         byte 0          ; number of active sprite registers (0 - SPRITEMAX)
                                ; SSS runtime variables:
sss             bytes 48        ; screen row index, computed from PLAYCOLS in SSSINIT
sssIDX          byte 0          ; current sprite loop index# (0-SPRITEMAX)
sssNUM          byte 0          ; current sprite being drawn# (0-SPRITEMAX)
sssX            byte 0          ; current sprite width: 0=8w, 1=16w, 2=24w
sssY            byte 0          ; current sprite height: 0=8h, 1=16h, 2=24h
sssBYTES        byte 0          ; number of bytes this sprite occupies
sssNEXT         byte 0          ; offset to adjacent character
sssCHAR         byte 0          ; next custom character to use on PENDING frame
sssDX           byte 0          ; delta X counter
sssDY           byte 0          ; delta Y counter
sssLINE         byte 0          ; current sprite make line: 0, 8, 16
sssLINENUM      byte 0          ; current sprite line countdown
sssROR1         byte 0          ; bit shift register column #1
sssROR2         byte 0          ; bit shift register column #2
ifdef SPRITEWIDE
sssROR3         byte 0          ; bit shift register column #3
endif
sssXFER         byte 0          ; transfer to custom character counter
sssMC           byte 0          ; is scurrent sprite MC?
sssSPRITEDDEF2  byte 0          ; store extended sprite definition
sssHEIGHTS      byte 8,16,24;
sssMCMASKS      byte 0, %11000000, %00110000, %00001100, %00000011


;user defined characters - room for 128 character definitions
;Note that if using any of the SSSPRINT procedures, characters 0 and 13
;from this area cannot be used because they conflict with the data terminator (0)
;and carriage return (13).

*=$1c00
       
;MULTICOLOUR FLOATING ISLAND
chars   BYTE    $00,$00,$00,$00,$00,$00,$00,$00
        BYTE    $00,$00,$00,$20,$20,$00,$00,$00
        BYTE    $00,$00,$20,$B8,$B8,$20,$00,$00
        BYTE    $00,$20,$B8,$B8,$B8,$B8,$20,$00
        BYTE    $00,$00,$00,$00,$00,$00,$00,$00
        BYTE    $44,$44,$10,$10,$44,$44,$00,$00
        BYTE    $00,$00,$00,$00,$00,$00,$00,$20
        BYTE    $00,$00,$00,$00,$00,$20,$B8,$B8
        BYTE    $20,$00,$00,$00,$00,$00,$00,$00
        BYTE    $B8,$B8,$20,$00,$00,$00,$00,$00
        BYTE    $00,$00,$00,$28,$AA,$AA,$AA,$FF
        BYTE    $00,$00,$00,$00,$00,$00,$CC,$FF
        BYTE    $00,$00,$10,$54,$54,$20,$EC,$FF
        BYTE    $00,$14,$55,$55,$28,$28,$F8,$FF
        BYTE    $14,$14,$0C,$CC,$3F,$3C,$CC,$FF
        BYTE    $20,$20,$B8,$20,$EC,$FC,$CC,$FF
        BYTE    $69,$69,$69,$BE,$AA,$BE,$AA,$55
        BYTE    $08,$08,$2E,$08,$2E,$2A,$8C,$AA
        BYTE    $00,$00,$0A,$2A,$A2,$AA,$5C,$FF
        BYTE    $0A,$22,$82,$AA,$04,$10,$5C,$FF
        BYTE    $00,$00,$80,$A0,$28,$A8,$D4,$FF
        BYTE    $80,$20,$08,$A8,$40,$10,$D4,$FF
        BYTE    $28,$A0,$A0,$A0,$28,$28,$CF,$FF
        BYTE    $05,$05,$15,$15,$15,$55,$DD,$FF
        BYTE    $50,$50,$50,$54,$54,$54,$DD,$FF
        BYTE    $FF,$3D,$3F,$0F,$00,$00,$00,$00
        BYTE    $FF,$FF,$FB,$EE,$FF,$0E,$0F,$03
        BYTE    $FA,$3A,$3E,$0F,$00,$00,$00,$00
        BYTE    $BB,$EA,$BE,$EF,$EA,$3E,$0F,$03
        BYTE    $AA,$AA,$AA,$AA,$AA,$AA,$AA,$AA
        BYTE    $00,$00,$00,$00,$02,$02,$09,$09
        BYTE    $AA,$BE,$AA,$BE,$69,$69,$69,$69
        BYTE    $00,$00,$00,$00,$80,$80,$60,$60
        BYTE    $EA,$3A,$EE,$3A,$FA,$EA,$FA,$EE
        BYTE    $03,$03,$0E,$0F,$3A,$3E,$3A,$FA
        BYTE    $BB,$AB,$AC,$BB,$AC,$AF,$AF,$AC
        BYTE    $FB,$EE,$AE,$EB,$BA,$AA,$AA,$AA
        BYTE    $BB,$FE,$EB,$BB,$AE,$AA,$AA,$AA
        BYTE    $AA,$AA,$BB,$AF,$FF,$80,$C0,$00
        BYTE    $AA,$AF,$EB,$FF,$30,$00,$00,$00
        BYTE    $AF,$BC,$FC,$C0,$00,$00,$00,$00
        BYTE    $FE,$EE,$FB,$AE,$BF,$F0,$C0,$00
        BYTE    $FF,$BC,$FC,$00,$00,$00,$00,$00
        BYTE    $00,$00,$00,$00,$00,$01,$55,$55
        BYTE    $C0,$C0,$B0,$F0,$AC,$BC,$AC,$AF
        BYTE    $AA,$AA,$AA,$BA,$EE,$BF,$FF,$F3
        BYTE    $55,$77,$FF,$EE,$BA,$AA,$AA,$AA
        BYTE    $C0,$F0,$A0,$BF,$AB,$AE,$AB,$AA
        BYTE    $00,$00,$00,$00,$F0,$B0,$FF,$BF
        BYTE    $EA,$EE,$EA,$BE,$EE,$BE,$AF,$BB
        BYTE    $AB,$BB,$AE,$BE,$BA,$BE,$EA,$FA
        BYTE    $00,$00,$00,$00,$0F,$3B,$3E,$EA
        BYTE    $03,$0F,$0E,$FE,$EA,$BA,$AA,$AA
        BYTE    $AA,$FA,$2E,$3A,$0E,$0F,$02,$03
        BYTE    $AA,$AF,$BB,$EC,$BC,$F0,$C0,$C0
        BYTE    $47,$45,$15,$15,$05,$05,$05,$05
        BYTE    $74,$D0,$50,$54,$54,$50,$50,$50
        BYTE    $F3,$3C,$F3,$0C,$00,$00,$00,$00
        BYTE    $03,$0F,$0F,$33,$3F,$3F,$F3,$CF
        BYTE    $FF,$3F,$FF,$FC,$FF,$FF,$CF,$FF
        BYTE    $C0,$F0,$C0,$FC,$3C,$F0,$FF,$F3
        BYTE    $FF,$3C,$F3,$3F,$CF,$FC,$3F,$FC
        BYTE    $CF,$FF,$0F,$3F,$33,$0F,$0F,$03
        BYTE    $FF,$F3,$3C,$FC,$C0,$F0,$C0,$00
        BYTE    $F0,$FA,$3A,$FC,$30,$C2,$CA,$28
        BYTE    $00,$80,$20,$20,$A0,$80,$00,$00
        BYTE    $20,$20,$20,$28,$08,$08,$08,$28
        BYTE    $00,$00,$41,$45,$55,$55,$14,$10
        BYTE    $28,$20,$20,$28,$0A,$02,$0A,$28
        BYTE    $00,$00,$00,$02,$0A,$A8,$A2,$00
        BYTE    $20,$A0,$80,$80,$0C,$2C,$8C,$00
        BYTE    $3C,$FE,$7C,$F0,$40,$02,$0A,$28
        BYTE    $04,$51,$15,$15,$55,$11,$41,$04
        BYTE    $03,$CC,$CC,$37,$04,$3B,$C8,$08
        BYTE    $00,$30,$C0,$00,$C0,$00,$C0,$30
        BYTE    $11,$01,$00,$40,$00,$10,$10,$00
        BYTE    $40,$04,$04,$40,$10,$10,$00,$00
        BYTE    $40,$40,$00,$01,$11,$10,$10,$10
        BYTE    $10,$10,$00,$04,$04,$44,$04,$00
        BYTE    $11,$45,$53,$17,$37,$54,$45,$11
        BYTE    $F5,$EF,$BE,$EB,$BA,$EE,$3E,$03
        BYTE    $00,$00,$00,$00,$00,$00,$05,$15
        BYTE    $00,$00,$00,$00,$00,$00,$40,$54
        BYTE    $5F,$FE,$BB,$EA,$AF,$FB,$BC,$C0
        BYTE    $54,$D4,$C4,$D1,$D4,$D4,$51,$45
        BYTE    $01,$10,$08,$40,$45,$11,$55,$55
        BYTE    $00,$30,$40,$70,$C0,$F0,$00,$00


;*********************************************************************
; Your main program code starts here
;

*=$2000
RESTART  
incasm "demo.asm"
incasm "VIC-SSS-MMX.asm"