;*********************************************************************
; DEMO featuring VIC Software Sprite Stack - MMX edition
; written by Robert Hurst <spock@hurst-ri.us>
; updated version: 19-Apr-2010.
; This demo will assemble using CBM prg Studio.
; The demo illustrates many of the features of the Software Sprite Stack,
; but specifically shows the use of:
;     24 Bit high sprites
;     multicolour and normal sprites
;     ghost mode, collision detection and repeat sprites
;
; Note that text strings need to be in the reversed character set.
; This project uses a custom character map (see Tools/Options/Assembler)
; to override the standard mapping.

; To assemble, include VIC-SSS-MMXH.asm and ssbuf.asm in that order in the
; Project Files build list.
; This demo file and the sprite library are included by ssbuf using incasm directives.
;*********************************************************************


               LDA  #$1e
               STA  VIC+$0F
               LDA  #6
               STA  COLORCODE
               JSR  SSSINIT        ; must occur BEFORE re-directing IRQ
;
; implement my interrupt vector
               SEI
               LDX  #<SSSIRQ
               LDY  #>SSSIRQ
               STX  $0314
               STY  $0315
               CLI



TITLE                              ;intro screen
               LDA  #$1a           ;white screen, reverse text and purple border
               STA  $900F
               lda  #$10           ;white auxiliary colour
               sta  $900e

               JSR  SSSPRINTS      ;print the following text using colours $F0 - F7
                                   ;note text specified using Reverse character codes
                                   ;13 is carriage return. String terminated by 0
               byte $F2,"SOFTWARE ",$F5,"SPRITE ",$F6,"STACK",13,13,0
               JSR  SSSPRINTS
               byte $F4,"  BY ROBERT HURST",13,13,0
               JSR  SSSPRINTS
               byte $F6,"MODIFIED BY BEAMRIDER",13,0
               JSR  SSSPRINTS
               byte $F6," TO GIVE 24 BIT HIGH ",13, "SINGLE AND MULTICOLOUR","       SPRITES",0
               JSR  SSSXYPRINTS
               BYTE 0,18,$F6,"MODIFIED BY THEGG TO",13, " ADD A DATA MAP LOADER",0

               ;set up our sprites
               lda  #%00000110     ; 24*16 sprite fixed
               sta  sssSPRITEDDEF2 ; new definition parameter for the new create
               LDA  #%10000000     ; turn sprite 0 on
               LDY  #20            ; height
               JSR  SSSCREATEEX    ; new create call
               LDA  #$0E           ; sprite colour- multicolour blue
               LDX  #<GHOST1       ; sprite definition
               LDY  #>GHOST1
               JSR  SSSANIM
               LDX  #$58
               ldy  #$60
               JSR  SSSMOVEXY

@begin         LDA #0
               STA  R0             ;set up animation frame count
@loop
               ldx  #0
               jsr  SSSUSE         ;signal using sprite 0
               ldx  R0             ; switch on frame count
               cpx  #0
               bne  @f1
               ldx  #<GHOST1
               ldy  #>GHOST1
               jmp  @anim
@f1            cpx #1
               bne  @f2
               ldx  #<GHOST2
               ldy  #>GHOST2
               jmp  @anim
@f2            cpx #2
               bne  @f3
               ldx  #<GHOST3
               ldy  #>GHOST3
               jmp  @anim
@f3            ldx #<GHOST4
               ldy  #>GHOST4
@anim          lda #$0e
               JSR  SSSANIM
               ldy  #10
               jsr  SSSFLIP        ;display the frame with a bit of delay


               JSR  ANYKEY         ;wait for user input
              ; BEQ  @begin
               bne  DEMO1
               inc  R0
               lda  R0
               cmp  #4
               bcc  @loop
               jmp  @begin

DEMO1
               LDA  #$1a           ;white screen and red border
               STA  $900F
               lda  #$50           ;aux colour green
               sta  $900e

               ;load screen map - new provision to load a screen map.
               ;Can use user defined characters loaded at $1c00 or the ROM
               ;based reversed characters. The colour options are to set the
               ;colour matrix to the value in the accumulator, or when Ac is $ff
               ; load a colour map.

               lda  #$0e           ;flag that the background is all blue
               jsr  SSSLOADBG
               word DataMap       ;address of the screen map
               ;word ColourMap    ;address of colour map when required.

               JSR  SSSPRINTS
               byte $F2,"SOFTWARE ",$F5,"SPRITE ",$F6,"STACK",13,13,0
               JSR  SSSPRINTS
               byte $F5,"  DATA MAP LOADING",0

               ;set up our sprite
               lda  #%00011101     ; 16*16 sprite floats X&Y
               sta  sssSPRITEDDEF2
               LDA  #%10000000     ; turn sprite on
               LDY  #$10           ; height
               JSR  SSSCREATEEX
               LDA  #$0e           ; multicolour blue
               LDX  #<BIGRED       ;create a new multicoloured sprite
               LDY  #>BIGRED
               JSR  SSSANIM
               LDX  #0             ;X position for sprite (must be even for multicolour)
               ldy  #$28
               JSR  SSSMOVEXY
                                   

@loop1         LDX #0
               JSR  SSSUSE
               ldx  SPRITEX        ;adjust X as X = X+2 for multicolour
               inx
               inx
               LDY  SPRITEY
               cpx  SSSCLIPX
               bne  @f1
               tya
               adc  #$20
               tay
               cpy  SSSCLIPY
               bcc  @f1
               ldy  #$28
@f1            JSR  SSSMOVEXY  ;move the MC sprite
               JSR  ANYKEY         ;until a key press
               BEQ  @loop1

DEMO2
               JSR  SSSPRINTS
               byte $F2,"SOFTWARE ",$F5,"SPRITE ",$F6,"STACK",13,13,0
               JSR  SSSPRINTS
               byte $F0," NORMAL AND ",$F3,"GHOST",13,0
               JSR  SSSPRINTS
               byte $F2,"                  ",13,13,0 ;print a colour change
               JSR  SSSPRINTS
               byte $F4,"AND A PROTECTED OBJECT",13,13,0
               JSR  SSSPRINTS
               byte $F6,"  SPRITE COLLISION",13,"  COLLIDED WITH: ",0

               ;set up our sprite
               lda  #%00010110     ; 24*16 sprite floats in the X axis
               sta  sssSPRITEDDEF2
               LDA  #%11000000     ; enable
               LDY  #$16           ; height
               JSR  SSSCREATEEX    ; create
               LDA  #$00           ; hires black
               LDX  #<BALOON       ; shape data
               LDY  #>BALOON
               JSR  SSSANIM
               LDX  #0             ; starting position
               ldy  #$20
               JSR  SSSMOVEXY

               ldx  #$20
               stx  R0

@loop           jsr BLOCK    ;display the protected cells
               ldx  #0
               jsr  SSSUSE
               LDX  R0
               ldy  #$20
               JSR  SSSMOVEXY
               LDA  SPRITEZ        ;control register
               AND  #$08           ;mask bit 3
               BEQ  @cont          ;no collision registered
               DEY                 ;collision so flash border
               STY  $900F
@cont           LDX #16        ;recover cursor position
               LDY  #9
               JSR  SSSPLOT
               LDA  #0
               STA  COLORCODE      ;set plot colour as black
               LDA  SPRITEBACK     ;get the ID of collision character or space
               JSR  SSSPRINT       ;print the character
               inc  R0
               lda  R0
               cmp  SSSCLIPX
               bne  @b1
               lda  SPRITEDEF      ;update the sprite definition
               eor  #%00100000     ;toggle between normal and ghost display
               sta  SPRITEDEF

@b1             jsr ANYKEY
               beq  @loop

DEMO3
               JSR  SSSPRINTS
               byte $F2,"SOFTWARE ",$F5,"SPRITE ",$F6,"STACK",13,13,0
               JSR  SSSXYPRINTS
               byte 0,2,$F5,"    REPEAT SPRITES",0

               ; Set up a string of multicolour sprites using the Repeat option
               lda  #%00010101     ; 16*16 sprite floats X&Y
               sta  sssSPRITEDDEF2
               LDA  #%10000000     ; turn sprite on
               LDY  #$10           ; height
               JSR  SSSCREATEEX    ; new create: Sprite 0
               LDA  #$0e           ; multicolour blue
               LDX  #<INVADE1      ;create a new multicoloured sprite
               LDY  #>INVADE1
               JSR  SSSANIM
               LDX  #$40           ;X position for sprite (must be even for multicolour)
               ldy  #$60
               JSR  SSSMOVEXY

               lda  #%00010101     ; 16*16 sprite floats X&Y
               sta  sssSPRITEDDEF2
               LDA  #%10010000     ; turn sprite on as a repeat of the last sprite
               LDY  #$10           ; height
               JSR  SSSCREATEEX    ; new create: Sprite 1
               lda  #$0d
               sta  SPRITECOL+1
               LDX  #$28           ;X position for sprite (must be even for multicolour)
               ldy  #$38
               JSR  SSSMOVEXY

               lda  #%00010101     ; 16*16 sprite floats X&Y
               sta  sssSPRITEDDEF2
               LDA  #%10010000     ; turn sprite on as a repeat of the last sprite
               LDY  #$10           ; height
               JSR  SSSCREATEEX    ; new create: Sprite 2
               lda  #$0c
               sta  SPRITECOL+2
               LDX  #$28           ;X position for sprite (must be even for multicolour)
               ldy  #$88
               JSR  SSSMOVEXY

               lda  #%00010101     ; 16*16 sprite floats X&Y
               sta  sssSPRITEDDEF2
               LDA  #%10010000     ; turn sprite on as a repeat of the last sprite
               LDY  #$10           ; height
               JSR  SSSCREATEEX    ; new create: Sprite 3
               lda  #$0c
               sta  SPRITECOL+3
               LDX  #$10           ;X position for sprite (must be even for multicolour)
               ldy  #$10
               JSR  SSSMOVEXY

               lda  #%00010101     ; 16*16 sprite floats X&Y
               sta  sssSPRITEDDEF2
               LDA  #%10010000     ; turn sprite on as a repeat of the last sprite
               LDY  #$10           ; height
               JSR  SSSCREATEEX    ; new create: Sprite 4
               lda  #$0c
               sta  SPRITECOL+4
               LDX  #$10           ;X position for sprite (must be even for multicolour)
               ldy  #$b0
               JSR  SSSMOVEXY

                                   ;Now the HiRes set
               lda  #%00010110     ; 24*16 sprite floats X&Y
               sta  sssSPRITEDDEF2
               LDA  #%10000000     ; enable
               LDY  #$16           ; height
               JSR  SSSCREATEEX    ; new create: Sprite 5
               LDA  #$00           ; hires black
               LDX  #<BALOON       ;
               LDY  #>BALOON
               JSR  SSSANIM
               LDX  #$b8           ;X position
               ldy  #$48
               JSR  SSSMOVEXY

               lda  #%00010110     ; 24*16 sprite floats X&Y
               sta  sssSPRITEDDEF2
               LDA  #%10010000     ; enable
               LDY  #$16           ; height
               JSR  SSSCREATEEX    ; new create: Sprite 6
               lda  #$02
               sta  SPRITECOL+6
               LDX  #$b8           ;X position
               ldy  #$20
               JSR  SSSMOVEXY

               lda  #%00010110     ; 24*16 sprite floats X&Y
               sta  sssSPRITEDDEF2
               LDA  #%10010000     ; enable
               LDY  #$16           ; height
               JSR  SSSCREATEEX    ; new create: Sprite 7
               lda  #$03
               sta  SPRITECOL+7
               LDX  #$b8           ;X position
               ldy  #$70
               JSR  SSSMOVEXY

               lda  #%00010110     ; 24*16 sprite floats X&Y
               sta  sssSPRITEDDEF2
               LDA  #%10010000     ; enable
               LDY  #$16           ; height
               JSR  SSSCREATEEX    ; new create: Sprite 8
               lda  #$04
               sta  SPRITECOL+8
               LDX  #$b8           ;X position
               ldy  #$98
               JSR  SSSMOVEXY
               ldx  #0
               stx  R1

@loop1         LDX #0
               stx  R0
               jsr  SSSUSE
               lda  R1
               beq  @f3
               dec  R1
               LDA  #$0e
               LDX  #<INVADE1      ;update animation
               LDY  #>INVADE1
               JMP  @f4
@f3            INC R1
               LDA  #$0e
               LDX  #<INVADE2      ;update animation
               LDY  #>INVADE2
@f4            JSR SSSANIM

@loop2         JSR SSSUSE
               lda  SPRITEX,x      ;
               LDY  SPRITEY,x
               tax
               lda  R0
               cmp  #5
               bcs  @f1
               inx
               inx
               jmp  @f2
@f1             dex
@f2             JSR SSSMOVEXY  ;move the sprite
               inc  R0
               ldx  R0
               cpx  #9
               bcc  @loop2
               jsr  ANYKEY
               beq  @loop1

FINI           JMP  RESET          ; bye-bye!
counter        byte 0
delay          byte 0

ANYKEY
               LDY  PLAYROWS
               DEY
               LDX  #0
               JSR  SSSPLOT
               JSR  SSSPRINTS
               byte $F6, "-=: PRESS ANY  KEY :=-"
               byte 0
                                   
@user          LDY #1
               JSR  SSSFLIP
               LDA  $C5
               CMP  #$40
               BEQ  @repeat        ; keypress?
@wait          LDA $C5
               CMP  #$40
               BNE  @wait          ; release key
               JSR  SSSINIT        ; start over - clears all sprite data
               LDA  #1
@repeat
               RTS


BLOCK          LDA  #0
               STA  COLORCODE      ;set colour
               LDY  #2
               LDX  #18
               LDA  #128+79
               JSR  SSSCELL        ;write protected cell
               LDY  #2
               LDX  #19
               LDA  #128+80
               JSR  SSSCELL
               LDY  #3
               LDX  #18
               LDA  #128+76
               JSR  SSSCELL        ;write character
               LDY  #3
               LDX  #19
               LDA  #128+122
               JSR  SSSCELL        ;write character
               RTS

INVADE1
               BYTE $03,$3F,$FF,$FF,$E3,$D7,$FF,$FF
               BYTE $0D,$0D,$30,$30,$C0,$C0,$00,$00
               BYTE $C0,$FC,$FF,$FF,$E3,$D7,$FF,$FF
               BYTE $70,$70,$0C,$0C,$03,$03,$00,$00
INVADE2
               BYTE $03,$3F,$FF,$FF,$CB,$D7,$FF,$3F
               BYTE $0C,$0D,$0D,$0C,$0C,$3C,$00,$00
               BYTE $C0,$FC,$FF,$FF,$CB,$D7,$FF,$FC
               BYTE $30,$70,$70,$30,$30,$3C,$00,$00


BIGRED         BYTE $05,$1F,$7F,$FF,$D7,$E7,$EB,$FF
               BYTE $FF,$FF,$F7,$DD,$FF,$FF,$CC,$44
               BYTE $50,$F4,$FD,$FF,$D7,$E7,$EB,$FF
               BYTE $FF,$FF,$DF,$77,$FF,$FF,$33,$11

GHOST1
               BYTE $01,$05,$15,$15,$55,$55,$7D,$6D
               BYTE $6D,$7D,$55,$55,$55,$55,$55,$55
               BYTE $55,$55,$44,$44,$00,$00,$00,$00
               BYTE $40,$50,$54,$54,$55,$55,$7D,$6D
               BYTE $6D,$7D,$55,$55,$55,$55,$55,$55
               BYTE $55,$55,$11,$11,$00,$00,$00,$00

GHOST2
               BYTE $01,$05,$15,$15,$55,$59,$79,$7D
               BYTE $7D,$7D,$55,$55,$55,$55,$55,$55
               BYTE $55,$55,$44,$44,$00,$00,$00,$00
               BYTE $40,$50,$54,$54,$55,$59,$79,$7D
               BYTE $7D,$7D,$55,$55,$55,$55,$55,$55
               BYTE $55,$55,$11,$11,$00,$00,$00,$00

GHOST3
               BYTE $01,$05,$15,$15,$55,$55,$7D,$79
               BYTE $79,$7D,$55,$55,$55,$55,$55,$55
               BYTE $55,$55,$44,$44,$00,$00,$00,$00
               BYTE $40,$50,$54,$54,$55,$55,$7D,$79
               BYTE $79,$7D,$55,$55,$55,$55,$55,$55
               BYTE $55,$55,$11,$11,$00,$00,$00,$00

GHOST4
               BYTE $01,$05,$15,$15,$55,$55,$7D,$7D
               BYTE $7D,$6D,$65,$55,$55,$55,$55,$55
               BYTE $55,$55,$44,$44,$00,$00,$00,$00
               BYTE $40,$50,$54,$54,$55,$55,$7D,$7D
               BYTE $7D,$6D,$65,$55,$55,$55,$55,$55
               BYTE $55,$55,$11,$11,$00,$00,$00,$00



BALOON
               BYTE $07,$1F,$3F,$3F,$7E,$7D,$7D,$7D
               BYTE $7E,$7F,$3F,$2F,$17,$13,$09,$09
               BYTE $05,$05,$07,$07,$07,$03,$00,$00
               BYTE $F0,$F8,$FC,$FC,$3E,$CE,$FE,$CE
               BYTE $3E,$FE,$FC,$F4,$E8,$C8,$90,$90
               BYTE $A0,$A0,$E0,$E0,$E0,$C0,$00,$00




;example screen map for plotting with SSSLOADBG

DataMap
               byte $01,$04,$04,$04,$03,$04,$02,$04,$01,$04,$00,$04,$03,$04,$02,$04
               byte $01,$04,$04,$04,$03,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$06,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$08
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$04,$04,$04,$02,$04,$04,$04,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$07,$04,$04
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$09,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$05,$04,$04,$3a,$3c
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$05,$04,$04
               byte $04,$04,$04,$04,$3d,$3d,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$3a,$3b,$3b,$3c,$04,$04,$04
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$3d
               byte $3b,$3b,$3d,$04,$04,$3a,$3c,$04,$04,$04,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$04,$3a,$3b,$3b,$3b,$3b,$3c,$04,$3d,$3d,$04,$04,$04
               byte $04,$3a,$3c,$04,$04,$04,$04,$04,$04,$04,$3e,$3b,$3b,$3b,$3b,$3f
               byte $04,$3d,$3d,$04,$04,$04,$3a,$3b,$3b,$3c,$04,$04,$04,$04,$04,$04
               byte $04,$39,$37,$38,$39,$04,$04,$3d,$3d,$04,$04,$04,$39,$37,$38,$39
               byte $04,$04,$04,$04,$0b,$0b,$0b,$0b,$17,$18,$0b,$0b,$0e,$17,$18,$0d
               byte $0c,$0b,$0b,$17,$18,$0b,$0b,$0b,$0b,$0b,$19,$1a,$24,$25,$24,$25
               byte $25,$24,$25,$25,$24,$25,$25,$25,$25,$24,$24,$25,$24,$25,$29,$2a
               byte $04,$04,$1b,$27,$1c,$31,$1d,$1d,$1d,$31,$32,$1d,$1d,$1d,$1d,$1d
               byte $31,$26,$27,$28,$04,$04,$04,$04,$04,$04,$04,$1b,$27,$28,$1b,$1c
               byte $26,$27,$27,$28,$1b,$27,$28,$04,$04,$04,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$02,$04
               byte $04,$04,$04,$04,$04,$04,$04,$04,$05,$04,$04,$04,$04,$04,$04,$04
               byte $04,$04,$04,$04

;example of a colour map plotted using SSSLOADBG

;ColourMap
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e

;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e,$0e
;               byte $0e,$0e,$0e,$0e




